/*  -*- c++ -*-
    
    $Id: driver.hpp 1513 2005-06-24 01:36:25Z yuk $
    $Date: 2005-06-24 10:36:25 +0900 (Fri, 24 Jun 2005) $
    $Rev: 1513 $
    $Author: yuk $
    
   */

#ifndef DRIVER_HPP_
#define DRIVER_HPP_

#include <iostream>
#include <sstream>
#include <iomanip>
#include "particles.h"
#include "dist.h"
#include "output.h"
#include "observable.hpp"
#include "mesh.h"
#include "utils.h"


template <class Model, // Particle model
	  class Method, // method of generating/updating pairtable
	  class Boundary, // boundary model
          class Random    // random number generator class
	  >
class Driver 
{
private:
  Particles* ptcls;
  Model* mdl;
  PairTable* ptbl;
  System* sys;
  Method* mthd;
  Boundary* bndry;
  Random* rnd;
  double hdt;
  double dt;
  int dtinv;
  
  //-----------------------------------------------------------
public:
  Driver(Particles& tptcls, Model& tmdl, 
         PairTable& tptbl, System& tsys,
         Method& tmeth, Boundary& tbndry,
         Random& trnd
         ){
    ptcls = &tptcls;
    mdl = &tmdl;
    ptbl = &tptbl;
    sys = &tsys;
    mthd = &tmeth;
    bndry = &tbndry;
  
    rnd = &trnd;
    
    dt = sys->dt;
    dtinv = sys->dtinv;
    
    hdt = 0.5*dt;
    
  }
  //--------------------------------------------------------------------------
  inline void sym2poststep(void){
    ptcls->clearforce();
    // force calculation
    calcforce();
    
    bndry->boundaryforce(*ptcls);
    
    for(int i=0;i<ptcls->number;i++){
      ptcls->px[i] += dt*ptcls->fx[i];
      ptcls->py[i] += dt*ptcls->fy[i];
      ptcls->pz[i] += dt*ptcls->fz[i];
    }
  }
  //--------------------------------------------------------------------------
  inline void sym2qstep(void){
    for(int i=0;i<ptcls->number;i++){
      ptcls->qx[i] += hdt*ptcls->px[i];
      ptcls->qy[i] += hdt*ptcls->py[i];
      ptcls->qz[i] += hdt*ptcls->pz[i];
    }
    bndry->boundarymove(*ptcls);
  }
  //--------------------------------------------------------------------------
  template <class Out>
  inline void updatePairTable(unsigned long& rme,Out& output){
    rme --;
    unsigned long curtime = lifetime();
    if (rme > curtime || rme < 3){	 // create
      rme = curtime;
//      *output.log << "regenerate table\n";
      mthd->generatePtable(*ptcls,*ptbl,*bndry);
    }
  }
  //--------------------------------------------------------------------------
  template <class Out>
  Driver& drive(double intrv,
                Out& output,
                double obint=0){
    unsigned long steps = static_cast<unsigned long>(intrv*dtinv);
    unsigned long estep = sys->globalstarttime + steps;
    unsigned long remtime = lifetime();
    unsigned long istep;
    
    int iobc = 0;
    int  iobint = static_cast<int>(obint*dtinv);
   

    // create
    mthd->generatePtable(*ptcls,*ptbl,*bndry);
    observation<Out>(0,output);
    for(istep=sys->globalstarttime;istep< estep; istep++){
      sym2qstep();
      updatePairTable<Out>(remtime,output);
      sym2poststep();
      sym2qstep();
      
      if(iobc == iobint){
        iobc=0;
        observation<Out>(istep,output);
      }
      iobc ++;
    }
    observation<Out>(istep,output);


    sys->globalstarttime = estep;
    
    return *this;
  }
  //--------------------------------------------------------
  template <class Out>
  inline void observation(unsigned long istep,
                          Out& output){
    double kene,pene;
    
    // pre-observation
    mthd->updatePtable(*ptcls,*ptbl,*bndry);
    // initialize 
    output.initialize(istep*dt);
    // main observation
    Observable::Energy<Model,Boundary>(*ptcls,*mdl,*ptbl,*bndry,
                              *output.out,kene,pene);
    // finalize
    output.finalize();
    output.flush();
  }
  //----------------------------------------------------------
  inline void calcforce(){
    for(int l=0;l<(*ptbl).pairnum;l++){
      int i = (*ptbl).idx1[l];
      int j = (*ptbl).idx2[l];
    
      double dxt = ptcls->qx[i]-ptcls->qx[j];
      double dyt = ptcls->qy[i]-ptcls->qy[j];
      double dzt = ptcls->qz[i]-ptcls->qz[j];
        
      bndry->canonicalize(dxt,dyt,dzt);
      double d2 = dxt*dxt+dyt*dyt+dzt*dzt;
      double ftmp = mdl->force(d2);
        
      ptcls->fx[i] += ftmp*dxt;
      ptcls->fx[j] -= ftmp*dxt;
        
      ptcls->fy[i] += ftmp*dyt; 
      ptcls->fy[j] -= ftmp*dyt;
        
      ptcls->fz[i] += ftmp*dzt;
      ptcls->fz[j] -= ftmp*dzt;

    }
  }
  
  inline unsigned long lifetime(){
    
    double vtmp = 0;
    for(int i=1;i<ptcls->number;i++){
      double tt2 = std::pow(ptcls->px[i],2)
        +std::pow(ptcls->py[i],2)
        +std::pow(ptcls->pz[i],2);
      if(tt2 > vtmp) vtmp = tt2;
    }
    vtmp = std::sqrt(vtmp);

    double criticaltime = ((*mthd).meshsize-(*mdl).cutoff)/(2.0*vtmp);
//    std::cout << vtmp << " " << criticaltime*dtinv<< std::endl;
    return static_cast<unsigned long>(criticaltime*dtinv);
    
  }
  
  
};
#endif // DRIVER_HPP_
