/* -*- c++ -*-
   $Id: main_global.cpp 1543 2005-06-29 07:22:58Z yuk $
   $Date: 2005-06-29 16:22:58 +0900 (Wed, 29 Jun 2005) $
   $Rev: 1543 $
   $Author: yuk $

*/

#include <iostream>
#include <iomanip>
#include "mtrand.h"

#include "geometry.h"
#include "particles.h"
#include "dist.h"
#include "system.hpp"
#include "particlemodels.hpp"
#include "mesh.h"
#include "observable.hpp"
#include "driver.hpp"
#include "boundary.hpp"
#include "output.h"

#include <sys/time.h>
#include <sys/resource.h>

#include <cstdio>

extern "C" {
//  新村さんの分
#include	"comApplController.h"
#include	"general.h"
#include	"my_mpi.h"  
#include "result.h"
}

#ifdef MPICODE
#include <mpi.h>
#endif // MPICODE

inline double my_clock()
{
  struct rusage RU;
  getrusage(RUSAGE_SELF, &RU);
  return RU.ru_utime.tv_sec + (double)RU.ru_utime.tv_usec*1e-6;
}

inline double my_wallclock(){
 struct timeval tp;
 struct timezone tzp;
 gettimeofday(&tp,&tzp);
 return tp.tv_sec + (double)tp.tv_usec*1e-6;
}
	
int main(int argc, char* argv[])
{
  OutPutFiles files(argc,argv);
  double score,tbsscore;

  double tt1,tt2,tt3,tt4;
  double td1,td2;
  int myid = 0;
  int numprocs = 1;
#ifdef MPICODE
  MPI::Init(argc,argv);
  numprocs = MPI::COMM_WORLD.Get_size();
  myid = MPI::COMM_WORLD.Get_rank();
#endif // MPICODE

  // ============= Yukawa routines ======================= 

  const int ms = 4;
  const double cutoff = 3.9;
  int num = 50000; 
  double inter = 0.05;
  double blen = 10*ms; 
  const double temp = 1.0;
  
//    std::cout << "Particle number " << num << " ";
  *files.log << "Particle number " << num << " ";
//    std::cout << "Particle density " << static_cast<double>(num)/(blen*blen*blen) 
//              << " ";
  *files.log << "Particle density " << static_cast<double>(num)/(blen*blen*blen) 
             << " ";
  
  unsigned long seed = 123456789;
  mtrand rnd(seed);
  
  LennardJones lj(cutoff);
  Particles ptcl1(num);
  System sys1(temp);   
  
//    std::cout << "Total steps " << inter*sys1.dtinv << "\n";
  *files.log << "Total steps " <<  inter*sys1.dtinv << "\n";
  
  PairTable pt1(num);
  Geometry geom(blen,blen,blen);
  PeriodicBoundary bndry(geom);
  Mesh   mesh1(geom,ms);
  //++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  ptcl1.initialize<mtrand>(geom,sys1,rnd);
  mesh1.generatePtable<PeriodicBoundary>(ptcl1,pt1,bndry);
  Driver<LennardJones,Mesh,PeriodicBoundary,mtrand> 
    driver(ptcl1,lj,pt1,sys1,mesh1 ,bndry,rnd);
#ifdef MPICODE
  MPI::COMM_WORLD.Barrier();
#endif //MPICODE
  if(myid ==0){
    tt1 = my_wallclock() ;
  }
#ifdef MPICODE
  MPI::COMM_WORLD.Barrier();
#endif // MPICODE
  double t1 = my_clock();
  driver.drive<OutPutFiles>(inter,files,inter/10.);
  double t2 = my_clock();
#ifdef MPICODE
  MPI::COMM_WORLD.Barrier();
#endif 
  if(myid ==0){
    tt2 = my_wallclock() ;
    *files.log << "wallclock1 " << tt2 - tt1 << std::endl;
    td1 = tt2-tt1;
  }
#ifdef MPICODE
  MPI::COMM_WORLD.Barrier();
#endif // MPICODE

//  std::cout << myid << "     MD: cpu time = " << t2-t1 << " sec ";
  *files.log << "cpu time = " << t2-t1 << " sec ";
  
//  std::cout << " ( " << (t2-t1)/(inter*sys1.dtinv) << " sec/step) ";
  *files.log << " ( " << (t2-t1)/(inter*sys1.dtinv) << " sec/step)";
  
  double kups = num * inter*sys1.dtinv / (t2-t1)/1000 ;
  score = kups / 118.4356;
  
//  std::cout << kups << " KUPS ";
  *files.log <<kups << " KUPS\n";
  
//  std::cout <<"Score " << score << "\n";
  *files.log <<"Score "<< score << "\n";
  
/*
  model name      : AMD Opteron(tm) Processor 248
  stepping        : 8
  cpu MHz         : 2205.051
  cache size      : 1024 KB
  
  average KUPS 118.4356 KUPS g++ 3.4 64bit -O3
  
*/

  char*	CFG_File_default = "perco_BM.cfg";
    
  FILE*	fp_cfg;
  int		flag_state = MYSUCCESS;
  char*	cfgFileName;
  int  my_rank=0, process_num=1;
  
  
  cpresult cpres;
    
  //===================================
  if( (fp_cfg = fopen(CFG_File_default,"r")) != NULL ){
    cfgFileName = CFG_File_default;
  } else{
    std::abort();
  }
  fclose(fp_cfg);
    

#ifdef MPICODE
  MPI::COMM_WORLD.Barrier();
#endif 
  if(myid==0) {
    tt3 = my_wallclock();
  }
#ifdef MPICODE
  MPI::COMM_WORLD.Barrier();
#endif // MPICODE
  flag_state = comApp_workWithConfigFile_para(argv[0], cfgFileName,
                                              my_rank, process_num,
                                              &cpres,
                                              myid
                                              );
#ifdef MPICODE  
  MPI::COMM_WORLD.Barrier();
#endif 
  if(myid ==0){
    tt4 = my_wallclock();
    *files.log << "wallclock2 " << tt4 - tt3 << std::endl;
    td2 = tt4-tt3;
  }
#ifdef MPICODE
  MPI::COMM_WORLD.Barrier();
#endif // MPICODE
/*
  model name      : AMD Opteron(tm) Processor 248
  stepping        : 8
  cpu MHz         : 2205.051
  cache size      : 1024 KB
  
  average TBS 4227.5329 TBS gcc 3.4 64bit  -O3
*/
    
  tbsscore = cpres.number/cpres.sec/4227.5329;
    
//  std::cout << "Score " << tbsscore << std::endl;
  *files.log << "Score " << tbsscore << std::endl;
    


//  std::cout << "MD Score: " << score;
  *files.log << "MD Score: " << score;
  
//  std::cout << " CP Score: " << tbsscore;
  *files.log << " CP Score: " << tbsscore;

//  std::cout << " Total Score " << (score + tbsscore)/2 << std::endl;
  *files.log << " Total Score " << (score + tbsscore)/2 << std::endl;

  if(myid ==0){
    kups = num * inter*sys1.dtinv / (td1)/1000 ;
    double mdscorempi = numprocs*kups / 118.4356;
    double tbsscorempi = numprocs*cpres.number/td2/4227.5329;
    std::cout << "Num of threads " << numprocs  ;
    std::cout << " MD thread score " << mdscorempi ;
    std::cout << " CP thread score " << tbsscorempi ;
    std::cout << " Total thread score " << (mdscorempi + tbsscorempi)/2 
              << std::endl;
              
    *files.log << "Num of threads " << numprocs  ;
    *files.log << " MD thread score " << mdscorempi ;
    *files.log << " CP thread score " << tbsscorempi ;
    *files.log << " Total thread score " << (mdscorempi + tbsscorempi)/2 
              << std::endl;
  }
  
  
  //////////////////////////////////////////////
#ifdef MPICODE
  MPI::Finalize();  
#endif 

  return 0;
}

