/*  -*- c++ -*-
    $Id: particles.h 1512 2005-06-24 00:57:06Z yuk $
    $Date: 2005-06-24 09:57:06 +0900 (Fri, 24 Jun 2005) $
    $Rev: 1512 $
    $Author: yuk $

*/

#ifndef PARTICLES_H_
#define PARTICLES_H_

#include <iostream>
#include "utils.h"
#include "geometry.h"
#include "system.hpp"
#include "mtrand.h"
#include "dist.h"

class Particles
{
private:
  
public:
  const int number; // total number


  // 3d coordinates
  double *qx;
  double *qy;
  double *qz;
  // 3d momenta
  double *px;
  double *py;
  double *pz;
  // 3d force
  double *fx;
  double *fy;
  double *fz;
//-----------------------------------------------------------------   
  // constructor 
  Particles(int num, std::ostream& output = std::cerr):
    number(num)
    {
      /*
      info<int>("Particles","Create Particles: number: ",number,output);
      */
      qx = new double [number];
      qy = new double [number];
      qz = new double [number];
      
      px = new double [number];
      py = new double [number];
      pz = new double [number];
      
      fx = new double [number];
      fy = new double [number];
      fz = new double [number];
      for(int i=0;i<number;i++){
        qx[i] = 0;
        qy[i] = 0;
        qz[i] = 0;
        px[i] = 0;
        py[i] = 0;
        pz[i] = 0;
        fx[i] = 0;
        fy[i] = 0;
        fz[i] = 0;
      }

    }
//-------------------------------------------------------------------
  ~Particles(void){
    delete [] qx;
    delete [] qy;
    delete [] qz;

    delete [] px;
    delete [] py;
    delete [] pz;

    delete [] fx;
    delete [] fy;
    delete [] fz;
  }
//----------------------------------------------------------------------
  // geometry で定義された領域にflagに応じてつめる。
  // system で定義されている温度に応じて、運動量を設定する。
  template <class T>
  Particles& initialize(Geometry& geom, // pack region
                        System& sys, 
                        T& rnd,
                        std::ostream& output = std::cerr
                        ){
    double x1 = 0;
    double x2 = geom.getX();
    double y1 = 0;
    double y2 = geom.getY();
    double z1 = 0;
    double z2 = geom.getZ();
    setPosition(number,0,1,x1,x2,y1,y2,z1,z2,output);

    // 速度の設定
    // 質量m 温度t のBoltzmann分布
    // 
    double temp = sys.getTemperature();
    // chamber内部
    for(int i=0;i<number;i++){
      px[i] = rnd.genrand_nd(temp);
    }
    for(int i=0;i<number;i++){
      py[i] = rnd.genrand_nd(temp);
    }
    for(int i=0;i<number;i++){
      pz[i] = rnd.genrand_nd(temp);
    }
      
    return *this;
  }
//----------------------------------------------------------------------   
  Particles& setPosition(int number,int offset,double a,
                         double x1,double x2,
                         double y1,double y2,
                         double z1,double z2,
                         std::ostream& output = std::cerr){
    // simple cubic
    double e0[]={0.5,0.5,0.5};
    double e1[]={1,0,0};
    double e2[]={0,1,0};
    // for special case ()
    double e3[]={0,0,a};
    double la = 1.0;
    for(int i=0;i<3;i++){
      e1[i] *= la;
      e2[i] *= la;
      e3[i] *= la;
    }
    pack(number,offset,qx,qy,qz,e0,e1,e2,e3,x1,x2,y1,y2,z1,z2,output);
      
    return *this;
  }
//----------------------------------------------------------------------
  double getKinEnergy(void){
    double ene = 0;
    for(int i=0;i<number;i++){
      ene += (px[i]*px[i]+py[i]*py[i]+pz[i]*pz[i]);
    }
    ene /= 2;
    ene /= static_cast<double>(number);
    return ene;
  }
//---------------------------------------------------------------------

  template <class Model>
  double getPotEnergy(Model& pmodel, PairTable& ptable){
    double ene = 0;
    for(int i=0;i<ptable.pairnum;i++){
      ene += pmodel.pairPotential(ptable.dist[i]);
    }
      
    return ene/static_cast<double>(number);
  }
//----------------------------------------------------------------------
  void clearforce(void){
    for(int i=0;i<number;i++){
      fx[i] = 0;
      fy[i] = 0;
      fz[i] = 0;
    }
  }
//----------------------------------------------------------------------

};

#endif // PARTICLES_H_
